package com.pj.tb_goods_transit;

import java.io.Serializable;
import java.util.Date;

import com.baomidou.mybatisplus.annotation.*;
import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import lombok.EqualsAndHashCode;

import lombok.Data;
import lombok.experimental.Accessors;

/**
 * Model: tb_goods_transit -- 审核商户在APP端发布的商品
 * @author qzy 
 */
@Data
@Accessors(chain = true)
@TableName(TbGoodsTransit.TABLE_NAME)
@EqualsAndHashCode(callSuper = false)
public class TbGoodsTransit extends Model<TbGoodsTransit> implements Serializable {

	// ---------- 模块常量 ----------
	/**
	 * 序列化版本id 
	 */
	private static final long serialVersionUID = 1L;	
	/**
	 * 此模块对应的表名 
	 */
	public static final String TABLE_NAME = "tb_goods_transit";	
	/**
	 * 此模块对应的权限码 
	 */
	public static final String PERMISSION_CODE = "tb-goods-transit";
	public static final String PERMISSION_CODE_ADD = "tb-goods-transit-add";
	public static final String PERMISSION_CODE_EDIT = "tb-goods-transit-edit";
	public static final String PERMISSION_CODE_DEL = "tb-goods-transit-del";




	// ---------- 表中字段 ----------
	/**
	 * 在销商品表主键 
	 */
	@TableId(type = IdType.AUTO)
	@JsonSerialize(using = ToStringSerializer.class)
	private Long id;	

	/**
	 * 商户编号 
	 */
	private String merchantId;	

	/**
	 * 商户名称 
	 */
	private String merchantName;	

	/**
	 * 商品类型 
	 */
	private String goodsType;	
	/**
	 * 商品图片
	 */
	private String goodsImg;

	/**
	 * 海关商品表主键
	 */
	private Long goodsId;

	/**
	 * 商品名称 
	 */
	private String goodsName;	

	/**
	 * 商品单位(吨、kg、个、柜) 
	 */
	private String goodsUnits;

	/** 互市区名称 */
	private String tradeAreaName;

	/** 互市区ID */
	private Long tradeAreaId;

	/**
	 * 净重
	 */
	private Double netWeight;

	/**
	 * 毛重
	 */
	private Double grossWeight;

	/**
	 * 商品价格 
	 */
	private Double price;

	/**
	 * 海关申报单号
	 */
	private String declareOdd;

	/**
	 * 产地(国家) 
	 */
	private String placeOrigin;	

	/**
	 * 商品描述 
	 */
	private String description;

	/**
	 * 库存数量 
	 */
	private Double stock;

	/**
	 * 商品状态(0=下架,1=在售) 
	 */
	private Integer goodsStatus;	

	/**
	 * 审核状态(0=待审核,1=通过,2=不通过) 
	 */
	private Integer auditStatus;	

	/**
	 * 审核时间 
	 */
	@JsonFormat(pattern = "yyyy-MM-dd")
	private Date auditTime;

	/**
	 * 审核意见 
	 */
	private String remark;	

	/**
	 * 创建时间 
	 */
	@JsonFormat(pattern = "yyyy-MM-dd")
	private Date createTime;

	/**
	 * 创建人编号 
	 */
	private String createBy;

	/**
	 * 创建人名称 
	 */
	private String createName;	

	/**
	 * 更新时间 
	 */
	@JsonFormat(pattern = "yyyy-MM-dd")
	private Date updateTime;

	/**
	 * 更新人编号 
	 */
	private String updateBy;	

	/**
	 * 更新人名称 
	 */
	private String updateName;
	/**
	 * 商铺名称
	 */
	private String shopName;
	/**
	 * 商铺ID
	 */
	private Long shopId;

	/**
	 * 删除状态(0=禁用,1=启用) 
	 */
	private Integer deleteStatus;

	/**
	 * 是否已被下单[0=未被下单,1=已被下单]
	 */
	private Integer isOrders;

	/**
	 * 商铺确认情况[0=待确认,1=已确认,2=拒绝]
	 */
	private Integer enterpriseConfirm;

	/**
	 * 是否已售罄(0=未售罄,1=已售罄)
	 */
	private Integer sellOut;



	


}
