package com.pj.tb_goods_units;

import java.io.IOException;
import java.util.List;
import com.pj.utils.so.SoMap;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import com.pj.utils.sg.*;
import com.pj.project4sp.SP;

import cn.dev33.satoken.annotation.SaCheckPermission;
import org.springframework.web.multipart.MultipartFile;


/**
 * Controller: tb_goods_units -- 商品单位
 * @author qzy 
 */
@RestController
@RequestMapping("/TbGoodsUnits/")
public class TbGoodsUnitsController {

	/** 底层 Service 对象 */
	@Autowired
	TbGoodsUnitsService tbGoodsUnitsService;

	/** 增 */  
	@RequestMapping("add")
	@SaCheckPermission(TbGoodsUnits.PERMISSION_CODE_ADD)
	public AjaxJson add(TbGoodsUnits t){
		tbGoodsUnitsService.add(t);
		t = tbGoodsUnitsService.getById(SP.publicMapper.getPrimarykey());
		return AjaxJson.getSuccessData(t);
	}

	/** 删 */  
	@RequestMapping("delete")
	@SaCheckPermission(TbGoodsUnits.PERMISSION_CODE_DEL)
	public AjaxJson delete(Long id){
		 tbGoodsUnitsService.delete(id);
		return AjaxJson.getSuccess();
	}
	
	/** 删 - 根据id列表 */  
	@RequestMapping("deleteByIds")
	@SaCheckPermission(TbGoodsUnits.PERMISSION_CODE_DEL)
	public AjaxJson deleteByIds(){
		List<Long> ids = SoMap.getRequestSoMap().getListByComma("ids", long.class); 
		int line = SP.publicMapper.deleteByIds(TbGoodsUnits.TABLE_NAME, ids);
		return AjaxJson.getByLine(line);
	}
	
	/** 改 */  
	@RequestMapping("update")
	@SaCheckPermission(TbGoodsUnits.PERMISSION_CODE_EDIT)
	public AjaxJson update(TbGoodsUnits t){
		tbGoodsUnitsService.update(t);
		return AjaxJson.getSuccess();
	}

	/** 查 - 根据id */  
	@RequestMapping("getById")
		@SaCheckPermission(TbGoodsUnits.PERMISSION_CODE)
	public AjaxJson getById(Long id){
		TbGoodsUnits t = tbGoodsUnitsService.getById(id);
		return AjaxJson.getSuccessData(t);
	}

	/** 查集合 - 根据条件（参数为空时代表忽略指定条件） */  
	@RequestMapping("getList")
		@SaCheckPermission(TbGoodsUnits.PERMISSION_CODE)
	public AjaxJson getList() { 
		SoMap so = SoMap.getRequestSoMap();
		List<TbGoodsUnits> list = tbGoodsUnitsService.getList(so.startPage());
		return AjaxJson.getPageData(so.getDataCount(), list);
	}


	/**
	 * 数据导入接口
	 * @param file
	 * @return
	 */
	@RequestMapping("goodsUnitsImport")
	@ResponseBody
	public AjaxJson dataImport(@RequestParam("file") MultipartFile file){
		try {
			String importData = tbGoodsUnitsService.importData(file);
			AjaxJson.getSuccess(importData);
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
		return AjaxJson.getError();
	}

	/**
	 * 导出
	 * @param keyword
	 * @return
	 */
	@RequestMapping("goodsUnitsOutport")
	@ResponseBody
	public AjaxJson dataOutport(@RequestParam(value = "keyword",required = false) String keyword,@RequestParam(value = "filepath",required = false)String filepath){
		try {
			String importData = tbGoodsUnitsService.outportExcel(keyword,filepath);
			AjaxJson.getSuccess(importData);
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
		return AjaxJson.getError();
	}








}
