package com.pj.tb_shop;

import java.io.IOException;
import java.util.List;

import com.pj.api.dto.ShopDto;
import com.pj.utils.so.SoMap;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import com.pj.utils.sg.*;
import com.pj.project4sp.SP;

import cn.dev33.satoken.annotation.SaCheckPermission;
import org.springframework.web.multipart.MultipartFile;


/**
 * Controller: tb_shop -- 商铺
 * @author qzy 
 */
@RestController
@RequestMapping("/TbShop/")
public class TbShopController {

	/** 底层 Service 对象 */
	@Autowired
	TbShopService tbShopService;

	/** 增 */  
	@RequestMapping("add")
	@SaCheckPermission(TbShop.PERMISSION_CODE_ADD)
	public AjaxJson add(TbShop t){
		tbShopService.add(t);
		t = tbShopService.getById(SP.publicMapper.getPrimarykey());
		return AjaxJson.getSuccessData(t);
	}

	/** 删 */  
	@RequestMapping("delete")
	@SaCheckPermission(TbShop.PERMISSION_CODE_DEL)
	public AjaxJson delete(Long id){
		 tbShopService.delete(id);
		return AjaxJson.getSuccess();
	}
	
	/** 删 - 根据id列表 */  
	@RequestMapping("deleteByIds")
	@SaCheckPermission(TbShop.PERMISSION_CODE_DEL)
	public AjaxJson deleteByIds(){
		List<Long> ids = SoMap.getRequestSoMap().getListByComma("ids", long.class); 
		int line = SP.publicMapper.deleteByIds(TbShop.TABLE_NAME, ids);
		return AjaxJson.getByLine(line);
	}
	
	/** 改 */  
	@RequestMapping("update")
	@SaCheckPermission(TbShop.PERMISSION_CODE_EDIT)
	public AjaxJson update(TbShop t){
		tbShopService.update(t);
		return AjaxJson.getSuccess();
	}

	/** 查 - 根据id */  
	@RequestMapping("getById")
		@SaCheckPermission(TbShop.PERMISSION_CODE)
	public AjaxJson getById(String id){
		TbShop t = tbShopService.getById(id);
		return AjaxJson.getSuccessData(t);
	}

	/** 查集合 - 根据条件（参数为空时代表忽略指定条件） */  
	@RequestMapping("getList")
		@SaCheckPermission(TbShop.PERMISSION_CODE)
	public AjaxJson getList() { 
		SoMap so = SoMap.getRequestSoMap();
		so.set("deleteStatus", 1);
		List<TbShop> list = tbShopService.getList(so.startPage());
		return AjaxJson.getPageData(so.getDataCount(), list);
	}
	
	
	
	/** 改 - 是否可用(0=否,1=是) */  
	@RequestMapping("updateStatus")
	@SaCheckPermission(TbShop.PERMISSION_CODE_EDIT)
	public AjaxJson updateStatus(String id, String value){
		int line = SP.publicMapper.updateColumnById(TbShop.TABLE_NAME, "status", value, id);
		return AjaxJson.getByLine(line);
	}





	/**
	 * 数据导入接口
	 * @param file
	 * @return
	 */
	@RequestMapping("shopImport")
	@ResponseBody
	public AjaxJson dataImport(@RequestParam("file") MultipartFile file){
		try {
			String importData = tbShopService.importData(file);
			AjaxJson.getSuccess(importData);
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
		return AjaxJson.getError();
	}

	/**
	 * 导出
	 * @param keyword
	 * @return
	 */
	@RequestMapping("shopOutport")
	@ResponseBody
	public AjaxJson dataOutport(@RequestParam(value = "keyword",required = false) String keyword,@RequestParam(value = "filepath",required = false)String filepath){
		try {
			String importData = tbShopService.outportExcel(keyword,filepath);
			AjaxJson.getSuccess(importData);
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
		return AjaxJson.getError();
	}


	/*------            正常业务👆👆    rpc远程调用👇👇                         -----------*/

	@RequestMapping("rpc/shopDto")
	public boolean shopDto(@RequestBody ShopDto shopDto) throws Exception {
		boolean b = tbShopService.shopDto(shopDto);
		return b;
	}


}
