package com.pj.tb_orders_cart;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.pj.project4sp.SP;
import com.pj.utils.sg.AjaxJson;
import com.pj.utils.so.SoMap;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;


/**
 * Controller: tb_orders_cart -- 订单购物车
 *
 * @author linbl
 */
@RestController
@RequestMapping("/TbOrdersCart/")
public class TbOrdersCartController {

    /**
     * 底层 Service 对象
     */
    @Autowired
    TbOrdersCartService tbOrdersCartService;

    /**
     * 增
     */
    @RequestMapping("add")
    @SaCheckPermission(TbOrdersCart.PERMISSION_CODE_ADD)
    public AjaxJson add(TbOrdersCart t) {
        tbOrdersCartService.add(t);
        t = tbOrdersCartService.getById(SP.publicMapper.getPrimarykey());
        return AjaxJson.getSuccessData(t);
    }

    /**
     * 删
     */
    @RequestMapping("delete")
    @SaCheckPermission(TbOrdersCart.PERMISSION_CODE_DEL)
    public AjaxJson delete(Long id) {
        tbOrdersCartService.delete(id);
        return AjaxJson.getSuccess();
    }

    /**
     * 删 - 根据id列表
     */
    @RequestMapping("deleteByIds")
    @SaCheckPermission(TbOrdersCart.PERMISSION_CODE_DEL)
    public AjaxJson deleteByIds() {
        List<Long> ids = SoMap.getRequestSoMap().getListByComma("ids", long.class);
        int line = SP.publicMapper.deleteByIds(TbOrdersCart.TABLE_NAME, ids);
        return AjaxJson.getByLine(line);
    }

    /**
     * 改
     */
    @RequestMapping("update")
    @SaCheckPermission(TbOrdersCart.PERMISSION_CODE_EDIT)
    public AjaxJson update(TbOrdersCart t) {
        tbOrdersCartService.update(t);
        return AjaxJson.getSuccess();
    }

    /**
     * 查 - 根据id
     */
    @RequestMapping("getById")
    @SaCheckPermission(TbOrdersCart.PERMISSION_CODE)
    public AjaxJson getById(String id) {
        TbOrdersCart t = tbOrdersCartService.getById(id);
        return AjaxJson.getSuccessData(t);
    }

    /**
     * 查集合 - 根据条件（参数为空时代表忽略指定条件）
     */
    @RequestMapping("getList")
    @SaCheckPermission(TbOrdersCart.PERMISSION_CODE)
    public AjaxJson getList() {
        SoMap so = SoMap.getRequestSoMap();
        List<TbOrdersCart> list = tbOrdersCartService.getList(so.startPage());
        return AjaxJson.getPageData(so.getDataCount(), list);
    }


}
